# -*- coding: utf-8 -*-
"""
code extracted from
https://rafaelbarreto.wordpress.com/2011/08/27/a-pyqt-widget-for-opencv-camera-preview/
"""
from PyQt4 import QtGui, QtCore
import numpy as np
import cv, cv2
        
__all__ = ["OpenCVQImage","CameraDevice","CameraWidget"]


class OpenCVQImage(QtGui.QImage):

    def __init__(self, rgbImg):

        assert rgbImg.ndim == 3 and rgbImg.shape[2] == 3, \
            "The input must be a 3-channel image."
        assert rgbImg.dtype == np.uint8, "The input must be an 8-bit image."

        self._imgData = rgbImg.tostring()
        h, w = rgbImg.shape[0:2]
        super(OpenCVQImage, self).__init__(self._imgData, w, h,
                                           QtGui.QImage.Format_RGB888)


class CameraDevice(QtCore.QObject):

    _DEFAULT_FPS = 50
    _DEFAULT_FRAME_SIZE = (640, 480)

    newFrame = QtCore.pyqtSignal(np.ndarray)

    def __init__(self, cameraId=0, mirrored=False, parent=None):
        super(CameraDevice, self).__init__(parent)

        self.mirrored = mirrored
        
        self._cameraDevice = cv2.VideoCapture(cameraId)
        if not self._cameraDevice.isOpened():
            raise IOError("Could not open camera '%d'." % cameraId)

        self.frameSize = self._DEFAULT_FRAME_SIZE

        self._timer = QtCore.QTimer(self)
        self._timer.timeout.connect(self._queryFrame)
        self._timer.setInterval(1000/self.fps)

        self.paused = False

    @QtCore.pyqtSlot()
    def _queryFrame(self):
        _, frame = self._cameraDevice.read()
        if frame is None:
            return
        if self.mirrored:
            frame = cv2.flip(frame, 1)
        # it's assumed the frame is in BGR format
        self.newFrame.emit(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))

    @property
    def paused(self):
        return not self._timer.isActive()

    @paused.setter
    def paused(self, p):
        if p:
            self._timer.stop()
        else:
            self._timer.start()

    @property
    def fps(self):
        fps = int(self._cameraDevice.get(cv.CV_CAP_PROP_FPS))
        if not fps > 0:
            fps = self._DEFAULT_FPS
        return fps

    @property
    def frameSize(self):
        w = self._cameraDevice.get(cv.CV_CAP_PROP_FRAME_WIDTH)
        h = self._cameraDevice.get(cv.CV_CAP_PROP_FRAME_HEIGHT)
        return int(w), int(h)

    @frameSize.setter
    def frameSize(self, newSize):
        w, h = newSize
        self._cameraDevice.set(cv.CV_CAP_PROP_FRAME_WIDTH, w)
        self._cameraDevice.set(cv.CV_CAP_PROP_FRAME_HEIGHT, h)

    # added by Thierry Bottaro. give a negative value for my Quickcam Pro 9000 (but somehow inversely linked to exposure)
    @property
    def exposureTime(self):
        expT = self._cameraDevice.get(cv.CV_CAP_PROP_EXPOSURE)
        return int(expT)

    # added by Thierry Bottaro. not working for my Quickcam Pro 9000 :(
    @exposureTime.setter
    def exposureTime(self, expT):
        self._cameraDevice.set(cv.CV_CAP_PROP_EXPOSURE, expT/1000.)

class CameraWidget(QtGui.QWidget):

    newFrame = QtCore.pyqtSignal(np.ndarray)

    def __init__(self, cameraDevice, parent=None):
        super(CameraWidget, self).__init__(parent)

        self._frame = None

        self._cameraDevice = cameraDevice
        self._cameraDevice.newFrame.connect(self._onNewFrame)

        w, h = self._cameraDevice.frameSize
        self.setMinimumSize(w, h)

    @QtCore.pyqtSlot(np.ndarray)
    def _onNewFrame(self, frame):
        # self._frame = frame.copy()
        # self.newFrame.emit(self._frame)
        self._frame = frame
        self.newFrame.emit(frame)
        self.update()

    def changeEvent(self, e):
        if e.type() == QtCore.QEvent.EnabledChange:
            if self.isEnabled():
                self._cameraDevice.newFrame.connect(self._onNewFrame)
            else:
                self._cameraDevice.newFrame.disconnect(self._onNewFrame)

    def paintEvent(self, e):
        # if self._frame is None:
        if self._frame is None:
            return
        painter = QtGui.QPainter(self)
        h, w = self._frame.shape[0:2]
        pt = QtCore.QPoint((self.width()-w)/2, (self.height()-h)/2)
        painter.drawImage(pt, OpenCVQImage(self._frame))    
